var bridgeCallbacks = {};

var update_loc = {update: 'Update', updating: 'updating'};

function getUpdateStr() {return update_loc.update;};
function getUpdatingStr() {return update_loc.updating;};

function executeBridgeCall(fn, onComplete) {
    var id = fn();
    if (id !== null) {
        bridgeCallbacks[fn().toString()] = onComplete;
    }
};

function processBridgeResponse(id, payload, errors) {
    id = id.toString();
    if (!bridgeCallbacks[id]) return;
    bridgeCallbacks[id]({payload: payload, errors: errors});
    delete bridgeCallbacks[id];
};

function getEnvVarA(lookup, fallback, onComplete) {
    function call() {
        if (window._egl_settings && window._egl_settings.getStringAsync) {
            return window._egl_settings.getStringAsync(lookup, fallback);
        } else {
            console.warn('Settings bridge or getStringAsync method missing');
            return null;
        }
    }
    executeBridgeCall(call, onComplete);
}

function isWebWorkerSupported() {
    return 'serviceWorker' in navigator;
}

function setOfflineLoc() {
    var appInitialized = isWebWorkerSupported();
    var postfix = appInitialized ? '_initialized' : '';
    var locFields = [
        'errorTitle',
        'errorLine1',
        'errorLine2',
        'button'
    ];

    setLocalizedText('OFFLINESCREEN', locFields, postfix);
}

function setReqUpdateLoc() {
    var locFields = [
        'selfUpdateTitle',
        'selfUpdateLine1',
        'updateButton'
    ]

    setLocalizedText('REQ_UPDATE', locFields, '');

    getEnvVarA('REQ_UPDATE.updateButton', 'error', function(result) {
        var locValue = result.payload;
        update_loc.update = locValue;
    });

    getEnvVarA('REQ_UPDATE.updateButton.updating', 'error', function(result) {
        var locValue = result.payload;
        update_loc.updating = locValue;
    });
}

function setLocalizedText(namespace, locFields, postfix) {
    for (var i = 0; i < locFields.length; i++) {
        var f = function() {
            var locField = locFields[i];
            var lookupKey = namespace + '.' + locField + postfix;
            var locClass = 'loc-' + locField;
            var elements = document.getElementsByClassName(locClass);
            getEnvVarA(lookupKey, 'error', function(result) {
                var locValue = result.payload;
                for (var k = 0; k < elements.length; k++) {
                   elements[k].innerHTML = locValue;
                }
            });
        };
        f();
    }
}

function reloadApp() {
    if (window.isFullyLoaded) {
        if (window._egl_activity && window._egl_activity.reloadAsync) {
            window._egl_activity.reloadAsync();
        } else {
            console.warn('Activity bridge or reloadAsync method missing');
        }
    }
}

function onResize() {
    setMaxWidth();
}

function setMaxWidth() {
    var content = document.getElementById('content');
    var contentRect = content.getBoundingClientRect();

    var maxWidth = contentRect.height < contentRect.width ? Math.min(contentRect.height * 1.2, contentRect.width) : contentRect.width;
    content.style.maxWidth = maxWidth + 'px';
}

function initOfflinePage() {
    setMaxWidth();
    setOfflineLoc();
    setTimeout(function() {
        window.isFullyLoaded = true;
    }, 0);
}

function initReqUpdatePage() {
    setMaxWidth();
    setReqUpdateLoc();
    setTimeout(function() {
        window.isFullyLoaded = true;
    }, 0);
}

function selfUpdate() {
    if (window._egl_library && window._egl_library.selfUpdateAsync) {
        window._egl_library.selfUpdateAsync();
    } else {
        console.warn("Library bridge or selfUpdateAsync method missing");
    }
}

function setUpdateButtonText(text) {
    var updateButton = document.getElementById('OfflineSelfUpdate_updateButton');
    updateButton.innerText = text;
}

var updatingTimeout;

function setUpdatingString() {
    var updateButton = document.getElementById('OfflineSelfUpdate_updateButton');
    var ELLIPSIS_LEN = '...'.length;

    if (updateButton.innerText === getUpdateStr()) {
        setUpdateButtonText(getUpdatingStr());
    } else {
        if (updateButton.innerText.length < getUpdatingStr().length + ELLIPSIS_LEN) {
            setUpdateButtonText(updateButton.innerText + '.');
        } else {
            setUpdateButtonText(getUpdatingStr());
        }
    }
}

function getErrorTitle(errorCode) {
    return errorMessages['epic.diesel.androidlauncher.alert.' + errorCode +'.title'] ||
        errorMessages['epic.diesel.androidlauncher.alert.default.title'];
}

function getErrorDesc(errorCode) {
    return errorMessages['epic.diesel.androidlauncher.alert.' + errorCode +'.description'] ||
        errorMessages['epic.diesel.androidlauncher.alert.default.description'];
}

function toggleContentOverlay(visible) {
    var content = document.getElementById('content');
    visible ? content.classList.add('alertOverlay') : content.classList.remove('alertOverlay');
}

function errorContentVisible(visible) {
   var errorContent = document.getElementById('alertContent');
   errorContent.style.visibility = visible ? 'visible' : 'hidden';
}

function handleError(errorCode) {
    var errorTitle = getErrorTitle(errorCode);
    var errorTitleElem = document.getElementById('alertTitle');
    errorTitleElem.innerText = errorTitle;

    var errorCodeElem = document.getElementById('alertCode');
    errorCodeElem.innerText = errorCode;

   var errorDesc = getErrorDesc(errorCode);
   var errorDescElem = document.getElementById('alertDesc');
   errorDescElem.innerText = errorDesc;

   var errorBtnOnClick = getErrorAckFn(errorCode);
   var errorBtn = document.getElementById('alertAction');
   errorBtn.onclick = errorBtnOnClick;

   errorContentVisible(true)

   toggleContentOverlay(true);

   setUpdateButtonText(getUpdateStr());
   clearTimeout(updatingTimeout);
}

function getErrorAckFn(errorCode) {
    return function () {
        if (errorCode === "LIBRARY-IN-NOPERM-INSTALL_NON_MARKET_APPS") {
            if (window._egl_activity && window._egl_activity.showAllowNonMarketAppsSettingAsync) {
                window._egl_activity.showAllowNonMarketAppsSettingAsync();
            } else {
                console.warn("Activity bridge or showAllowNonMarketAppsSettingAsync method missing");
            }
        }
        toggleContentOverlay(false);
        errorContentVisible(false);

        var updateButton = document.getElementById('OfflineSelfUpdate_updateButton');
        updateButton.innerText = getUpdateStr();
    }
}

function getLauncherAppA(onComplete) {
    function call() {
        if (window._egl_library && window._egl_library.getAppAsync) {
            return window._egl_library.getAppAsync('', '', 'EpicGamesLauncher');
        } else {
            console.warn("Library bridge or getAppAsync method missing");
            return null;
        }
    }
    executeBridgeCall(call, onComplete);
}

function appHasTasks(app) {
    return !!(app && app.tasks.length);
}

function pollLauncherStatus() {
    getLauncherAppA(function(result) {
        var launcher = result.payload;
        if (appHasTasks(launcher)) {
                setUpdatingString();
            } else {
                setUpdateButtonText(getUpdateStr());
            }
            setTimeout(pollLauncherStatus, 200);
    });
}

window.onmessage = function(message) {
    if (message.type === 'ASYNC_CALL_RESPONSE') {
        processBridgeResponse(message.id, message.payload, message.error);
    } else if (message.payload && message.payload.error) {
        handleError(message.payload.error);
    }
}
